@@ -22,7 +22,7 @@ def laplace_filter(gray_img, ksize, scale):
     lp_filtered = laplacian filtered image
 
     :param gray_img: numpy.ndarray
-    :param kernel: int
+    :param ksize: int
     :param scale: int
     :return lp_filtered: numpy.ndarray
     """
@@ -30,7 +30,8 @@ def laplace_filter(gray_img, ksize, scale):
     lp_filtered = cv2.Laplacian(src=gray_img, ddepth=-1, ksize=ksize, scale=scale)
     params.device += 1
     if params.debug == 'print':
-        print_image(lp_filtered, os.path.join(params.debug_outdir,
+        print_image(lp_filtered,
+                    os.path.join(params.debug_outdir,
                                  str(params.device) + '_lp_out_k' + str(ksize) + '_scale' + str(scale) + '.png'))
     elif params.debug == 'plot':
         plot_image(lp_filtered, cmap='gray')
