@@ -1,15 +1,14 @@
 # Color Corrections Functions
 
 import os
-import errno
 import cv2
 import numpy as np
-from plantcv.plantcv import print_image
+from plantcv.plantcv import params
+from plantcv.plantcv import outputs
 from plantcv.plantcv import plot_image
-from plantcv.plantcv import fatal_error
-from plantcv.plantcv import params
 from plantcv.plantcv.roi import circle
-from plantcv.plantcv import params
+from plantcv.plantcv import print_image
+from plantcv.plantcv import fatal_error
 
 
 def get_color_matrix(rgb_img, mask):
@@ -406,10 +405,10 @@ def quick_color_check(target_matrix, source_matrix, num_chips):
     over saturated color chips or other issues.
 
     Inputs:
-    source_matrix      = a 22x4 matrix containing the average red value, average green value, and
-                             average blue value for each color chip of the source image
-    target_matrix      = a 22x4 matrix containing the average red value, average green value, and
-                             average blue value for each color chip of the target image
+    source_matrix      = an nrowsXncols matrix containing the avg red, green, and blue values for each color chip
+                            of the source image
+    target_matrix      = an nrowsXncols matrix containing the avg red, green, and blue values for each color chip
+                            of the target image
     num_chips          = number of color card chips included in the matrices (integer)
 
     :param source_matrix: numpy.ndarray
@@ -476,19 +475,22 @@ def quick_color_check(target_matrix, source_matrix, num_chips):
             print(p1)
 
 
-def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurry=False, background='dark'):
+def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurry=False, background='dark',
+                    record_chip_size="median"):
     """Automatically detects a color card and output info to use in create_color_card_mask function
 
     Algorithm written by Brandon Hurr. Updated and implemented into PlantCV by Haley Schuhl.
 
-    Inputs:
-    rgb_img        = Input RGB image data containing a color card.
-    threshold      = Threshold method, either 'normal', 'otsu', or 'adaptgauss', optional (default 'adaptgauss')
-    threshvalue    = Thresholding value, optional (default 125)
-    blurry         = Bool (default False) if True then image sharpening applied
-    background     = Type of image background either 'dark' or 'light' (default 'dark'); if 'light' then histogram
+        Inputs:
+    rgb_img          = Input RGB image data containing a color card.
+    threshold_type   = Threshold method, either 'normal', 'otsu', or 'adaptgauss', optional (default 'adaptgauss')
+    threshvalue      = Thresholding value, optional (default 125)
+    blurry           = Bool (default False) if True then image sharpening applied
+    background       = Type of image background either 'dark' or 'light' (default 'dark'); if 'light' then histogram
                         expansion applied to better detect edges, but histogram expansion will be hindered if there
                         is a dark background
+    record_chip_size = Optional str for choosing chip size measurement to be recorded, either "median",
+                        "mean", or None
 
     Returns:
     df             = Dataframe containing information about the filtered contours
@@ -496,10 +498,11 @@ def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurr
     spacing        = Two element tuple of spacing between centers of chips
 
     :param rgb_img: numpy.ndarray
-    :param threshold: str
+    :param threshold_type: str
     :param threshvalue: int
     :param blurry: bool
     :param background: str
+    :param record_chip_size: str
     :return df: pandas.core.frame.DataFrame
     :return start_coord: tuple
     :return spacing: tuple
@@ -511,11 +514,11 @@ def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurr
 
     # Get image attributes
     height, width, channels = rgb_img.shape
-    totalpx = float(height * width)
+    total_pix = float(height * width)
 
     # Minimum and maximum square size based upon 12 MP image
-    minarea = 1000. / 12000000. * totalpx
-    maxarea = 8000000. / 12000000. * totalpx
+    min_area = 1000. / 12000000. * total_pix
+    max_area = 8000000. / 12000000. * total_pix
 
     # Create gray image for further processing
     gray_img = cv2.cvtColor(rgb_img, cv2.COLOR_BGR2GRAY)
@@ -544,28 +547,28 @@ def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurr
         fatal_error('Background parameter ' + str(background) + ' is not "light" or "dark"!')
 
     # Thresholding
-    if threshold_type == "otsu":
+    if threshold_type.upper() == "OTSU":
         # Blur slightly so defects on card squares and background patterns are less likely to be picked up
         gaussian = cv2.GaussianBlur(gray_img, (5, 5), 0)
         ret, threshold = cv2.threshold(gaussian, 0, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)
-    elif threshold_type == "normal":
+    elif threshold_type.upper() == "NORMAL":
         # Blur slightly so defects on card squares and background patterns are less likely to be picked up
         gaussian = cv2.GaussianBlur(gray_img, (5, 5), 0)
         ret, threshold = cv2.threshold(gaussian, threshvalue, 255, cv2.THRESH_BINARY)
-    elif threshold_type == "adaptgauss":
+    elif threshold_type.upper() == "ADAPTGAUSS":
         # Blur slightly so defects on card squares and background patterns are less likely to be picked up
         gaussian = cv2.GaussianBlur(gray_img, (11, 11), 0)
         threshold = cv2.adaptiveThreshold(gaussian, 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C,
                                           cv2.THRESH_BINARY_INV, 51, 2)
     else:
-        fatal_error('Threshold ' + str(threshold_type) + ' is not "otsu", "normal", or "adaptgauss"!')
+        fatal_error('Input threshold_type=' + str(threshold_type) + ' but should be "otsu", "normal", or "adaptgauss"!')
 
     # Apply automatic Canny edge detection using the computed median
-    edges = skimage.feature.canny(threshold)
-    edges.dtype = 'uint8'
+    canny_edges = skimage.feature.canny(threshold)
+    canny_edges.dtype = 'uint8'
 
     # Compute contours to find the squares of the card
-    contours, hierarchy = cv2.findContours(edges, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)[-2:]
+    contours, hierarchy = cv2.findContours(canny_edges, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)[-2:]
     # Variable of which contour is which
     mindex = []
     # Variable to store moments
@@ -612,19 +615,23 @@ def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurr
     # Loop over our contours and extract data about them
     for index, c in enumerate(contours):
         # Area isn't 0, but greater than min-area and less than max-area
-        if marea[index] != 0 and minarea < marea[index] < maxarea:
+        if marea[index] != 0 and min_area < marea[index] < max_area:
             peri = cv2.arcLength(c, True)
             approx = cv2.approxPolyDP(c, 0.1 * peri, True)
             center, wh, angle = cv2.minAreaRect(c)  # Rotated rectangle
             mwidth.append(wh[0])
             mheight.append(wh[1])
-            mwhratio.append(wh[0] / wh[1])
+            # In different versions of OpenCV, width and height can be listed in a different order
+            # To normalize the ratio we sort them and take the ratio of the longest / shortest
+            wh_sorted = list(wh)
+            wh_sorted.sort()
+            mwhratio.append(wh_sorted[1] / wh_sorted[0])
             msquare.append(len(approx))
             # If the approx contour has 4 points then we can assume we have 4-sided objects
             if len(approx) == 4 or len(approx) == 5:
                 msquarecoords.append(approx)
             else:  # It's not square
-                #msquare.append(0)
+                # msquare.append(0)
                 msquarecoords.append(0)
         else:  # Contour has area of 0, not interesting
             msquare.append(0)
@@ -634,16 +641,16 @@ def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurr
             mwhratio.append(0)
 
     # Make a pandas df from data for filtering out junk
-    locarea = {'index': mindex, 'X': mx, 'Y': my, 'width': mwidth, 'height': mheight, 'WHratio': mwhratio,
-               'Area': marea, 'square': msquare, 'child': mchild}
-    df = pd.DataFrame(locarea)
+    all_contours = {'index': mindex, 'x': mx, 'y': my, 'width': mwidth, 'height': mheight, 'res_ratio': mwhratio,
+                    'area': marea, 'square': msquare, 'child': mchild}
+    df = pd.DataFrame(all_contours)
 
     # Add calculated blur factor to output
     df['blurriness'] = blurfactor
 
     # Filter df for attributes that would isolate squares of reasonable size
-    df = df[(df['Area'] > minarea) & (df['Area'] < maxarea) & (df['child'] != -1) &
-            (df['square'].isin([4, 5])) & (df['WHratio'] < 1.2) & (df['WHratio'] > 0.85)]
+    df = df[(df['area'] > min_area) & (df['area'] < max_area) & (df['child'] != -1) &
+            (df['square'].isin([4, 5])) & (df['res_ratio'] < 1.2) & (df['res_ratio'] > 0.85)]
 
     # Filter nested squares from dataframe, was having issues with median being towards smaller nested squares
     df = df[~(df['index'].isin(df['index'] + 1))]
@@ -655,7 +662,7 @@ def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurr
     # Squares that are within 6 widths of the current square
     pixeldist = median_sq_width_px * 6
     # Computes euclidean distance matrix for the x and y contour centroids
-    distmatrix = pd.DataFrame(squareform(pdist(df[['X', 'Y']])))
+    distmatrix = pd.DataFrame(squareform(pdist(df[['x', 'y']])))
     # Add up distances that are less than  ones have distance less than pixeldist pixels
     distmatrixflat = distmatrix.apply(lambda dist: dist[dist <= pixeldist].count() - 1, axis=1)
 
@@ -663,7 +670,7 @@ def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurr
     df = df.assign(distprox=distmatrixflat.values)
 
     # Compute how similar in area the squares are. lots of similar values indicates card isolate area measurements
-    filtered_area = df['Area']
+    filtered_area = df['area']
     # Create empty matrix for storing comparisons
     sizecomp = np.zeros((len(filtered_area), len(filtered_area)))
     # Double loop through all areas to compare to each other
@@ -681,7 +688,7 @@ def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurr
     df = df.assign(sizeprox=sizematrix.values)
 
     # Reorder dataframe for better printing
-    df = df[['index', 'X', 'Y', 'width', 'height', 'WHratio', 'Area', 'square', 'child',
+    df = df[['index', 'x', 'y', 'width', 'height', 'res_ratio', 'area', 'square', 'child',
              'blurriness', 'distprox', 'sizeprox']]
 
     # Loosely filter for size and distance (relative size to median)
@@ -698,7 +705,7 @@ def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurr
     # Squares that are within 6 widths of the current square
     pixeldist = median_sq_width_px * 5
     # Computes euclidean distance matrix for the x and y contour centroids
-    distmatrix = pd.DataFrame(squareform(pdist(df[['X', 'Y']])))
+    distmatrix = pd.DataFrame(squareform(pdist(df[['x', 'y']])))
     # Add up distances that are less than  ones have distance less than pixeldist pixels
     distmatrixflat = distmatrix.apply(lambda dist: dist[dist <= pixeldist].count() - 1, axis=1)
 
@@ -708,25 +715,24 @@ def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurr
     # Filter results for distance proximity to other squares
     df = df[(df['distprox'] >= 4)]
     # Remove all not numeric values use to_numeric with parameter, errors='coerce' - it replace non numeric to NaNs:
-    df['X'] = pd.to_numeric(df['X'], errors='coerce')
-    df['Y'] = pd.to_numeric(df['Y'], errors='coerce')
+    df['x'] = pd.to_numeric(df['x'], errors='coerce')
+    df['y'] = pd.to_numeric(df['y'], errors='coerce')
 
     # Remove NaN
     df = df.dropna()
 
-
-    if df['X'].min() is np.nan or df['Y'].min() is np.nan:
+    if df['x'].min() is np.nan or df['y'].min() is np.nan:
         fatal_error('No color card found under current parameters')
     else:
         # Extract the starting coordinate
-        start_coord = (df['X'].min(), df['Y'].min())
+        start_coord = (df['x'].min(), df['y'].min())
 
         # start_coord = (int(df['X'].min()), int(df['Y'].min()))
         # Calculate the range
-        spacingx_short = (df['X'].max() - df['X'].min()) / 3
-        spacingy_short = (df['Y'].max() - df['Y'].min()) / 3
-        spacingx_long = (df['X'].max() - df['X'].min()) / 5
-        spacingy_long = (df['Y'].max() - df['Y'].min()) / 5
+        spacingx_short = (df['x'].max() - df['x'].min()) / 3
+        spacingy_short = (df['y'].max() - df['y'].min()) / 3
+        spacingx_long = (df['x'].max() - df['x'].min()) / 5
+        spacingy_long = (df['y'].max() - df['y'].min()) / 5
         # Chip spacing since 4x6 card assumed
         spacing_short = min(spacingx_short, spacingy_short)
         spacing_long = max(spacingx_long, spacingy_long)
@@ -734,4 +740,31 @@ def find_color_card(rgb_img, threshold_type='adaptgauss', threshvalue=125, blurr
         spacing = int(max(spacing_short, spacing_long))
         spacing = (spacing, spacing)
 
+    if record_chip_size is not None:
+        if record_chip_size.upper() == "MEDIAN":
+            chip_size = df.loc[:, "area"].median()
+            chip_height = df.loc[:, "height"].median()
+            chip_width = df.loc[:, "width"].median()
+        elif record_chip_size.upper() == "MEAN":
+            chip_size = df.loc[:, "area"].mean()
+            chip_height = df.loc[:, "height"].mean()
+            chip_width = df.loc[:, "width"].mean()
+        else:
+            print(str(record_chip_size) + " Is not a valid entry for record_chip_size." +
+                  " Must be either 'mean', 'median', or None.")
+            chip_size = None
+            chip_height = None
+            chip_width = None
+        # Store into global measurements
+        outputs.add_observation(variable='color_chip_size', trait='size of color card chips identified',
+                                method='plantcv.plantcv.transform.find_color_card', scale='none',
+                                datatype=float, value=chip_size, label=str(record_chip_size))
+        method = record_chip_size.lower()
+        outputs.add_observation(variable=f'{method}_color_chip_height', trait=f'{method} height of color card chips identified',
+                                method='plantcv.plantcv.transform.find_color_card', scale='none',
+                                datatype=float, value=chip_height, label=str(record_chip_size))
+        outputs.add_observation(variable=f'{method}_color_chip_width', trait=f'{method} size of color card chips identified',
+                                method='plantcv.plantcv.transform.find_color_card', scale='none',
+                                datatype=float, value=chip_width, label=str(record_chip_size))
+
     return df, start_coord, spacing
