@@ -3,5 +3,6 @@ from plantcv.plantcv.visualize.colorize_masks import colorize_masks
 from plantcv.plantcv.visualize.histogram import histogram
 from plantcv.plantcv.visualize.clustered_contours import clustered_contours
 from plantcv.plantcv.visualize.colorspaces import colorspaces
+from plantcv.plantcv.visualize.auto_threshold_methods import auto_threshold_methods
 
-__all__ = ["pseudocolor", "colorize_masks", "histogram", "clustered_contours", "colorspaces"]
+__all__ = ["pseudocolor", "colorize_masks", "histogram", "clustered_contours", "colorspaces", "auto_threshold_methods"]
